import java.awt.image.BufferedImage;

import org.json.JSONArray;
import org.json.JSONObject;

public class SaveData {
	private boolean inProgress = false;
	private int secondsplayed = 0;
	private String workshopID = "";
	private int state = 0; //?
	private CellType[] cellStates;
	
	private int width, height;
	private CellType[][] cellStates2d = null;
	
	public SaveData(JSONObject object) {
		this.inProgress = object.getBoolean("inProgress");
		this.secondsplayed = object.getInt("secondsPlayed");
		this.workshopID = object.getString("id");
		this.state = object.getInt("state");
		JSONArray jsonCellStates = object.getJSONArray("cellStates");
		this.cellStates = new CellType[jsonCellStates.length()];
		for (int i = 0; i < jsonCellStates.length(); ++i) {
			this.cellStates[i] = CellType.getByNumber(jsonCellStates.getInt(i));
		}
	}
	
	public void initDimensions(int width, int height) {
		if (width * height != this.cellStates.length) throw new IllegalArgumentException("invalid dimension for " + (width * height) + " cells");
		this.width = width;
		this.height = height;
		
		this.cellStates2d = new CellType[width][height];
		
		for (int i = 0; i < this.cellStates.length; ++i) {
			int y = i % height;
			int x = (int)Math.floor(i/height);
			this.cellStates2d[x][y] = this.cellStates[i];
		}
	}
	
//	public BufferedImage getCurrentProgressImage(float zoom) {
//		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
//		BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
//		for (int x = 0; x < width; x++) {
//		    for (int y = 0; y < height; y++) {
//		    	Color color = null;
//		    	switch(this.cellStates2d[x][y]) {
//					case EMPTY:
//						color = Color.white;
//						break;
//					case MARKED:
//						color = Color.blue;
//						break;
//					case MARKED_AS_EMPTY:
//						color = Color.lightGray;
//						break;
//					case MARKED_AS_FILLED:
//						color = Color.black;
//						break;
//		    	}
//		        bufferedImage.setRGB(x, y, color.getRGB());
//		    }
//		}
//		
//		return zoomImage(bufferedImage, zoom);
//	}
	
	public BufferedImage getCurrentProgressImage(float zoom) {
		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
		return ImageBuilder.createPixelImage(this.cellStates2d, zoom);
	}
	
	public BufferedImage getCurrentProgressImageGrid() {
		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
		return ImageBuilder.createGridImage(this.cellStates2d);
	}
	
	public BufferedImage getErrorImage(boolean[][] correctSolution, float zoom) {
		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
		return ImageBuilder.createPixelImage(getErrorCheckedCellStates(correctSolution), zoom);
	}
	
	public BufferedImage getErrorImageGrid(boolean[][] correctSolution) {
		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
		return ImageBuilder.createGridImage(getErrorCheckedCellStates(correctSolution));
	}

	public boolean isInProgress() {
		return inProgress;
	}

	public int getSecondsplayed() {
		return secondsplayed;
	}

	public String getWorkshopID() {
		return workshopID;
	}

	public int getState() {
		return state;
	}

	public CellType[] getCellStates() {
		return cellStates;
	}

	/**
	 * only if initialized with the dimensions!
	 * @return 
	 */
	public CellType[][] getCellStates2d() {
		return cellStates2d;
	}
	
	public CellType[][] getErrorCheckedCellStates(boolean[][] correctSolution) {
		if (this.cellStates2d == null) throw new IllegalArgumentException("initialize dimensions first");
		CellType[][] markedCellTypes = new CellType[width][height];
		for (int x = 0; x < width; x++) {
		    for (int y = 0; y < height; y++) {
		    	boolean cellShouldBeMarked = correctSolution[x][y];
		    	switch(this.cellStates2d[x][y]) {
					case MARKED_AS_EMPTY:
						markedCellTypes[x][y] = cellShouldBeMarked ? CellType.ERRORENOUS_MARKED_AS_EMPTY : CellType.MARKED_AS_EMPTY;
						break;
					case MARKED_AS_FILLED:
						markedCellTypes[x][y] = cellShouldBeMarked ? CellType.MARKED_AS_FILLED : CellType.ERRORENOUS_MARKED_AS_FILLED;
						break;
					default:
						markedCellTypes[x][y] = this.cellStates2d[x][y];
						break;
		    	}
		    }
		}
		return markedCellTypes;
	}
	
}
